@@ -19,17 +19,11 @@ struct CharInfo {
   unsigned int length:       4;
   unsigned int group:        1;
   unsigned int invoke:       1;
-  bool isKindOf(CharInfo c) { return type & c.type; }
+  CharInfo() : type(0), default_type(0), length(0), group(0), invoke(0) {}
+  bool isKindOf(CharInfo c) const { return type & c.type; }
 };
 
 class CharProperty {
- private:
-  scoped_ptr<Mmap<char> >   cmmap_;
-  std::vector<const char *>  clist_;
-  const CharInfo            *map_;
-  int                        charset_;
-  whatlog                    what_;
-
  public:
   bool open(const Param &);
   bool open(const char*);
@@ -84,8 +78,15 @@ class CharProperty {
 
   static bool compile(const char *, const char *, const char*);
 
-  explicit CharProperty(): cmmap_(new Mmap<char>), map_(0), charset_(0) {}
+  CharProperty(): cmmap_(new Mmap<char>), map_(0), charset_(0) {}
   virtual ~CharProperty() { this->close(); }
+
+ private:
+  scoped_ptr<Mmap<char> >   cmmap_;
+  std::vector<const char *>  clist_;
+  const CharInfo            *map_;
+  int                        charset_;
+  whatlog                    what_;
 };
 }
 #endif   // MECAB_CHARACTER_CATEGORY_H_
