@@ -23,15 +23,13 @@
 #include "config.h"
 #endif
 
-// tricky macro for MSVC
 #if defined(_MSC_VER) || defined(__CYGWIN__)
-#define for if (0); else for
 #define NOMINMAX
 #define snprintf _snprintf
 #endif
 
 #define COPYRIGHT "MeCab: Yet Another Part-of-Speech and Morphological Analyzer\n\
-\nCopyright(C) 2001-2011 Taku Kudo \nCopyright(C) 2004-2008 Nippon Telegraph and Telephone Corporation\n"
+\nCopyright(C) 2001-2012 Taku Kudo \nCopyright(C) 2004-2008 Nippon Telegraph and Telephone Corporation\n"
 
 #define SYS_DIC_FILE            "sys.dic"
 #define UNK_DEF_FILE            "unk.def"
@@ -45,6 +43,8 @@
 #define LEFT_ID_FILE            "left-id.def"
 #define RIGHT_ID_FILE           "right-id.def"
 #define POS_ID_FILE             "pos-id.def"
+#define MODEL_DEF_FILE          "model.def"
+#define MODEL_FILE              "model.bin"
 #define DICRC                   "dicrc"
 #define BOS_KEY                 "BOS/EOS"
 
@@ -54,8 +54,6 @@
 #define UNK_DEF_DEFAULT           "DEFAULT,0,0,0,*\nSPACE,0,0,0,*\n"
 #define MATRIX_DEF_DEFAULT        "1 1\n0 0 0\n"
 
-#define VERY_SMALL_LOGPROB      0.0001
-
 #ifdef MECAB_USE_UTF8_ONLY
 #define MECAB_DEFAULT_CHARSET "UTF-8"
 #endif
