@@ -1,6 +1,5 @@
 // MeCab -- Yet Another Part-of-Speech and Morphological Analyzer
 //
-//  $Id: context_id.cpp 173 2009-04-18 08:10:57Z taku-ku $;
 //
 //  Copyright(C) 2001-2006 Taku Kudo <taku@chasen.org>
 //  Copyright(C) 2004-2006 Nippon Telegraph and Telephone Corporation
@@ -16,7 +15,7 @@ using namespace MeCab;
 bool open_map(const char *filename,
               std::map<std::string, int> *cmap,
               Iconv *iconv) {
-  std::ifstream ifs(filename);
+  std::ifstream ifs(WPATH(filename));
   CHECK_DIE(ifs) << "no such file or directory: " << filename;
   cmap->clear();
   char *col[2];
@@ -29,7 +28,7 @@ bool open_map(const char *filename,
     if (iconv) {
       iconv->convert(&pos);
     }
-    cmap->insert(std::make_pair<std::string, int>
+    cmap->insert(std::pair<std::string, int>
                  (pos, std::atoi(col[0])));
   }
   return true;
@@ -47,7 +46,7 @@ bool build(std::map<std::string, int> *cmap,
 
 bool save(const char* filename,
           std::map<std::string, int> *cmap) {
-  std::ofstream ofs(filename);
+  std::ofstream ofs(WPATH(filename));
   CHECK_DIE(ofs) << "permission denied: " << filename;
   for (std::map<std::string, int>::const_iterator it = cmap->begin();
        it != cmap->end(); ++it) {
