@@ -11,7 +11,6 @@
 #include "scoped_ptr.h"
 #include "tokenizer.h"
 #include "utils.h"
-#include "scoped_ptr.h"
 #include "viterbi.h"
 
 namespace MeCab {
@@ -40,33 +39,36 @@ void inline read_node_info(const Dictionary &dic,
 
 template class Tokenizer<Node, Path>;
 template class Tokenizer<LearnerNode, LearnerPath>;
-
-#if __GNUC__ >= 4
 template Tokenizer<Node, Path>::Tokenizer();
 template void Tokenizer<Node, Path>::close();
 template const DictionaryInfo
 *Tokenizer<Node, Path>::dictionary_info() const;
 template Node* Tokenizer<Node, Path>::getBOSNode(Allocator<Node, Path> *) const;
 template Node* Tokenizer<Node, Path>::getEOSNode(Allocator<Node, Path> *) const;
-template Node* Tokenizer<Node, Path>::lookup(const char*,
-                                             const char*,
-                                             Allocator<Node, Path> *) const;
+template Node* Tokenizer<Node, Path>::lookup<false>(
+    const char *,
+    const char *,
+    Allocator<Node, Path> *,
+    Lattice *) const;
+template Node* Tokenizer<Node, Path>::lookup<true>(
+    const char *,
+    const char *,
+    Allocator<Node, Path> *,
+    Lattice *) const;
 template bool Tokenizer<Node, Path>::open(const Param &);
 template Tokenizer<LearnerNode, LearnerPath>::Tokenizer();
 template void Tokenizer<LearnerNode, LearnerPath>::close();
 template const DictionaryInfo
 *Tokenizer<LearnerNode, LearnerPath>::dictionary_info() const;
-template LearnerNode* Tokenizer<LearnerNode, LearnerPath>::getEOSNode(
+template LearnerNode * Tokenizer<LearnerNode, LearnerPath>::getEOSNode(
     Allocator<LearnerNode, LearnerPath> *) const;
-template LearnerNode* Tokenizer<LearnerNode, LearnerPath>::getBOSNode(
-    Allocator<LearnerNode, LearnerPath> *) const;
-template LearnerNode*
-Tokenizer<LearnerNode, LearnerPath>::lookup(
-    const char*,
-    const char*,
+template LearnerNode * Tokenizer<LearnerNode, LearnerPath>::getBOSNode(
     Allocator<LearnerNode, LearnerPath> *) const;
+template LearnerNode *Tokenizer<LearnerNode, LearnerPath>::lookup<false>(
+    const char *,
+    const char *,
+    Allocator<LearnerNode, LearnerPath> *, Lattice *) const;
 template bool Tokenizer<LearnerNode, LearnerPath>::open(const Param &);
-#endif
 
 template <typename N, typename P>
 Tokenizer<N, P>::Tokenizer()
@@ -97,18 +99,15 @@ bool Tokenizer<N, P>::open(const Param &param) {
 
   const std::string prefix = param.template get<std::string>("dicdir");
 
-  const char *mode = param.template get<bool>("open-mutable-dictionary") ?
-      "r+" : "r";
-
   CHECK_FALSE(unkdic_.open(create_filename
-                                 (prefix, UNK_DIC_FILE).c_str(), mode))
+                                 (prefix, UNK_DIC_FILE).c_str()))
       << unkdic_.what();
   CHECK_FALSE(property_.open(param)) << property_.what();
 
   Dictionary *sysdic = new Dictionary;
 
   CHECK_FALSE(sysdic->open
-                    (create_filename(prefix, SYS_DIC_FILE).c_str(), mode))
+                    (create_filename(prefix, SYS_DIC_FILE).c_str()))
       << sysdic->what();
 
   CHECK_FALSE(sysdic->type() == 0)
@@ -120,16 +119,16 @@ bool Tokenizer<N, P>::open(const Param &param) {
   const std::string userdic = param.template get<std::string>("userdic");
   if (!userdic.empty()) {
     scoped_fixed_array<char, BUF_SIZE> buf;
-    scoped_fixed_array<char *, BUF_SIZE> _dic;
+    scoped_fixed_array<char *, BUF_SIZE> dicfile;
     std::strncpy(buf.get(), userdic.c_str(), buf.size());
-    size_t n = tokenizeCSV(buf.get(), _dic.get(), _dic.size());
+    const size_t n = tokenizeCSV(buf.get(), dicfile.get(), dicfile.size());
     for (size_t i = 0; i < n; ++i) {
       Dictionary *d = new Dictionary;
-      CHECK_FALSE(d->open(_dic[i], mode)) << d->what();
+      CHECK_FALSE(d->open(dicfile[i])) << d->what();
       CHECK_FALSE(d->type() == 1)
-          << "not a user dictionary: " << _dic[i];
+          << "not a user dictionary: " << dicfile[i];
       CHECK_FALSE(sysdic->isCompatible(*d))
-          << "incompatible dictionary: " << _dic[i];
+          << "incompatible dictionary: " << dicfile[i];
       dic_.push_back(d);
     }
   }
@@ -152,10 +151,10 @@ bool Tokenizer<N, P>::open(const Param &param) {
   unk_tokens_.clear();
   for (size_t i = 0; i < property_.size(); ++i) {
     const char *key = property_.name(i);
-    Dictionary::result_type n = unkdic_.exactMatchSearch(key);
+    const Dictionary::result_type n = unkdic_.exactMatchSearch(key);
     CHECK_FALSE(n.value != -1) << "cannot find UNK category: " << key;
     const Token *token = unkdic_.token(n);
-    size_t size  = unkdic_.token_size(n);
+    size_t size = unkdic_.token_size(n);
     unk_tokens_.push_back(std::make_pair(token, size));
   }
 
@@ -180,6 +179,55 @@ bool Tokenizer<N, P>::open(const Param &param) {
   return true;
 }
 
+namespace {
+inline bool partial_match(const char *f1, const char *f2) {
+  if (std::strcmp(f1, "*") == 0) {
+    return true;
+  }
+
+  scoped_fixed_array<char, BUF_SIZE> buf1;
+  scoped_fixed_array<char, BUF_SIZE> buf2;
+  scoped_fixed_array<char *, 64> c1;
+  scoped_fixed_array<char *, 64> c2;
+
+  std::strncpy(buf1.get(), f1, buf1.size());
+  std::strncpy(buf2.get(), f2, buf2.size());
+
+  const size_t n1 = tokenizeCSV(buf1.get(), c1.get(), c1.size());
+  const size_t n2 = tokenizeCSV(buf2.get(), c2.get(), c2.size());
+  const size_t n  = std::min(n1, n2);
+
+  for (size_t i = 0; i < n; ++i) {
+    if (std::strcmp(c1[i], "*") != 0 &&
+        std::strcmp(c1[i], c2[i]) != 0) {
+      return false;
+    }
+  }
+
+  return true;
+}
+
+template <typename N>
+bool is_valid_node(const Lattice *lattice,  N *node) {
+  const size_t end_pos = node->surface - lattice->sentence() + node->length;
+  if (lattice->boundary_constraint(end_pos) == MECAB_INSIDE_TOKEN) {
+    return false;
+  }
+  const size_t begin_pos =
+      node->surface - lattice->sentence() + node->length - node->rlength;
+  const char *feature = lattice->feature_constraint(begin_pos);
+  if (!feature) {
+    return true;
+  }
+  if (lattice->boundary_constraint(begin_pos) == MECAB_TOKEN_BOUNDARY &&
+      lattice->boundary_constraint(end_pos) == MECAB_TOKEN_BOUNDARY &&
+      partial_match(feature, node->feature)) {
+    return true;
+  }
+  return false;
+}
+}  // namespace
+
 #define ADDUNKNWON do {                                                  \
     const Token *token = unk_tokens_[cinfo.default_type].first;          \
     size_t size  = unk_tokens_[cinfo.default_type].second;               \
@@ -190,20 +238,33 @@ bool Tokenizer<N, P>::open(const Param &param) {
       new_node->surface = begin2;                                        \
       new_node->length = begin3 - begin2;                                \
       new_node->rlength = begin3 - begin;                                \
-      new_node->bnext = result_node;                                     \
       new_node->stat = MECAB_UNK_NODE;                                   \
+      new_node->bnext = result_node;                                     \
       if (unk_feature_.get()) new_node->feature = unk_feature_.get();    \
+      if (isPartial && !is_valid_node(lattice, new_node)) { continue; }  \
       result_node = new_node; } } while (0)
 
 template <typename N, typename P>
+template <bool isPartial>
 N *Tokenizer<N, P>::lookup(const char *begin, const char *end,
-                           Allocator<N, P> *allocator) const {
+                           Allocator<N, P> *allocator, Lattice *lattice) const {
   CharInfo cinfo;
   N *result_node = 0;
   size_t mblen = 0;
   size_t clen = 0;
 
   end = static_cast<size_t>(end - begin) >= 65535 ? begin + 65535 : end;
+
+  if (isPartial) {
+    const size_t begin_pos = begin - lattice->sentence();
+    for (size_t n = begin_pos + 1; n < lattice->size(); ++n) {
+      if (lattice->boundary_constraint(n) == MECAB_TOKEN_BOUNDARY) {
+        end = lattice->sentence() + n;
+        break;
+      }
+    }
+  }
+
   const char *begin2 = property_.seekToOtherType(begin, end, space_,
                                                  &cinfo, &mblen, &clen);
 
@@ -216,19 +277,20 @@ N *Tokenizer<N, P>::lookup(const char *begin, const char *end,
         begin2,
         static_cast<size_t>(end - begin2),
         daresults, results_size);
-
     for (size_t i = 0; i < n; ++i) {
       size_t size = (*it)->token_size(daresults[i]);
       const Token *token = (*it)->token(daresults[i]);
       for (size_t j = 0; j < size; ++j) {
         N *new_node = allocator->newNode();
         read_node_info(**it, *(token + j), &new_node);
-//        new_node->token = (Token *)(token + j);
         new_node->length = daresults[i].length;
         new_node->rlength = begin2 - begin + new_node->length;
         new_node->surface = begin2;
         new_node->stat = MECAB_NOR_NODE;
         new_node->char_type = cinfo.default_type;
+        if (isPartial && !is_valid_node(lattice, new_node)) {
+          continue;
+        }
         new_node->bnext = result_node;
         result_node = new_node;
       }
@@ -244,7 +306,9 @@ N *Tokenizer<N, P>::lookup(const char *begin, const char *end,
 
   if (begin3 > end) {
     ADDUNKNWON;
-    return result_node;
+    if (result_node) {
+      return result_node;
+    }
   }
 
   if (cinfo.group) {
@@ -278,6 +342,34 @@ N *Tokenizer<N, P>::lookup(const char *begin, const char *end,
     ADDUNKNWON;
   }
 
+  if (isPartial && !result_node) {
+    begin3 = begin2;
+    while (true) {
+      cinfo = property_.getCharInfo(begin3, end, &mblen);
+      begin3 += mblen;
+      if (begin3 > end ||
+          lattice->boundary_constraint(begin3 - lattice->sentence())
+          != MECAB_INSIDE_TOKEN) {
+        break;
+      }
+    }
+    ADDUNKNWON;
+
+    if (!result_node) {
+      N *new_node = allocator->newNode();
+      new_node->char_type = cinfo.default_type;
+      new_node->surface = begin2;
+      new_node->length = begin3 - begin2;
+      new_node->rlength = begin3 - begin;
+      new_node->stat = MECAB_UNK_NODE;
+      new_node->bnext = result_node;
+      new_node->feature =
+          lattice->feature_constraint(begin - lattice->sentence());
+      CHECK_DIE(new_node->feature);
+      result_node = new_node;
+    }
+  }
+
   return result_node;
 }
 
